﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using techanto.Models;
using System.Globalization;
using System.Web.Configuration;



namespace techanto.Controllers
{
    public class BlogController : Controller
    {
        private TechantoContext db = new TechantoContext();


        //
        // GET: /Blog/ e
        //      /Blog/Index/1

        public ActionResult Index(int numPage)
        {
          
            LoadCommonViewBagStuff();           
            var posts = db.Posts.Include("Author").Include("Tags").Where(p => p.Published == true).OrderByDescending(t => t.Time);

            int numPostsPerPage = int.Parse(WebConfigurationManager.AppSettings["NumPostsPerPage"]);
            int numPages = (int)Math.Ceiling( Convert.ToDouble(posts.Count()) / numPostsPerPage);
                    
            ViewBag.NumPages = numPages;
            ViewBag.PageNumber = numPage;

         

            return View("Index", posts.Skip(numPostsPerPage * (numPage - 1)).Take(numPostsPerPage).ToList());
        }

        //
        // GET: /Blog/Details/5
        

        public ViewResult Details(int id)
        {
            LoadCommonViewBagStuff();
            Post post = db.Posts.Include("Author").Include("Tags").Where(p => p.Published == true).Single(p => p.Id == id);
            return View(post);
        }


        //
        // GET: /Blog/SearchTag/1/5
        //                      {numPage}/{id}  

        public ActionResult SearchTag(int numPage,int id)
        {
            LoadCommonViewBagStuff();


            var posts = (from post in db.Posts.Include("Author").Include("Tags")
                        where post.Tags.Where(t=> t.Id == id).Count() > 0 && post.Published == true
                        orderby post.Time
                        select post);


            int pageNumber = numPage;
            int numPostsPerPage = int.Parse(WebConfigurationManager.AppSettings["NumPostsPerPage"]);
            int numPages = (int)Math.Ceiling(Convert.ToDouble(posts.Count()) / numPostsPerPage);

            ViewBag.NumPages = numPages;
            ViewBag.PageNumber = pageNumber;
            ViewBag.CurrentTag = id;

            return View("Index",posts.Skip(numPostsPerPage * (pageNumber - 1)).Take(numPostsPerPage).ToList());

        }

        //
        // GET: /Blog/About

        public ActionResult About()
        {
            LoadCommonViewBagStuff();
            return View();
        }



        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        protected void LoadCommonViewBagStuff()
        {   
            //tornare solo i tags per cui esiste almeno un post 
            var tags = from tag in db.Tags
                       where (
                                from post in db.Posts.Include("Tags")
                                where post.Tags.Contains(tag) && post.Published == true
                                select post
                            ).Any()
                       select tag;

            ViewBag.Tags = tags.ToList();
        }
    }
}
