﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using techanto.Models;

namespace techanto.Controllers
{ 
    [Authorize]
    public class PostController : Controller
    {
        private TechantoContext db = new TechantoContext();

        //
        // GET: /Post/

        public ViewResult Index()
        {
            var posts = db.Posts.Include("Author");
            return View(posts.ToList());
        }

        //
        // GET: /Post/Details/5

        public ViewResult Details(int id)
        {
            Post post = db.Posts.Include("Tags").Include("Author").Single(p => p.Id == id);
            return View(post);
        }

        //
        // GET: /Post/Create

        public ActionResult Create()
        {
            ViewBag.Author_Id = new SelectList(db.Authors, "UserId", "UserName");
            return View();
        } 

        //
        // POST: /Post/Create

        [HttpPost]
        public ActionResult Create(Post post)
        {
            post.Time = DateTime.Now;
            if (ModelState.IsValid)
            {
                if ((post.TagsCommaSeparated != null) && (post.TagsCommaSeparated.Length > 0))
                {
                    //lista di tags da associare al post
                    List<String> tagsList = post.TagsCommaSeparated.Split(',').Select(t => t.Trim()).ToList<String>();
                 
                    //lista di tutti i tags presenti attualmente nel DB
                    List<String> allTags = db.Tags.Select(t => t.Value).ToList<String>();

                    //lista dei tags da creare (aggiuntivi)
                    var tagToCreate = tagsList.Where(t => !allTags.Contains(t) ).ToList<String>();

                    //crea i tag nuovi non ancora presenti nel DB
                    foreach (var tag in tagToCreate)
                    {
                        Tag tagObject = db.Tags.CreateObject();
                        tagObject.Value = tag;
                        db.Tags.AddObject(tagObject);
                    }
                    db.SaveChanges();

                    //recupera tutti gli oggetti Tag da associare al post (ora dovrebero essere tutti presenti nel DB             
                    //associate all the tags to the post
                    foreach(var tag in db.Tags.Where(t => tagsList.Contains(t.Value)))
                    {
                         post.Tags.Add(tag);
                    }
                   
                    
                }

                db.Posts.AddObject(post);
                db.SaveChanges();
                return RedirectToAction("Index");  
            }

            ViewBag.Author_Id = new SelectList(db.Authors, "UserId", "UserName", post.Author_Id);
            return View(post);
        }
        
        //
        // GET: /Post/Edit/5
 
        public ActionResult Edit(int id)
        {
            Post post = db.Posts.Include("Tags").Single(p => p.Id == id);
            ViewBag.Author_Id = new SelectList(db.Authors, "UserId", "UserName", post.Author_Id);
            return View(post);
        }

        //
        // POST: /Post/Edit/5

        [HttpPost]
        public ActionResult Edit(Post newPost)
        {
            if (ModelState.IsValid)
            {

                Post post = db.Posts.Include("Author").Include("Tags").Single(p => p.Id == newPost.Id);
                post.Title = newPost.Title;
                post.Time = newPost.Time;
                post.Author = db.Authors.Single(a => a.UserId == newPost.Author_Id); 
                post.Content = newPost.Content;
                post.Tags.Clear();
                post.TagsCommaSeparated = newPost.TagsCommaSeparated;
                post.Published = newPost.Published;
           
                if ((post.TagsCommaSeparated != null) && (post.TagsCommaSeparated.Length > 0))
                {
                    //lista di tags da associare al post
                    List<String> tagsList = post.TagsCommaSeparated.Split(',').Select(t => t.Trim()).ToList<String>();

                    //lista di tutti i tags presenti attualmente nel DB
                    List<String> allTags = db.Tags.Select(t => t.Value).ToList<String>();

                    //lista dei tags da creare (aggiuntivi)
                    var tagToCreate = tagsList.Where(t => !allTags.Contains(t)).ToList<String>();

                    //crea i tag nuovi non ancora presenti nel DB
                    foreach (var tag in tagToCreate)
                    {
                        Tag tagObject = db.Tags.CreateObject();
                        tagObject.Value = tag;
                        db.Tags.AddObject(tagObject);
                    }
                    db.SaveChanges();

                    //recupera tutti gli oggetti Tag da associare al post (ora dorebero essere tutti presenti nel DB             
                    //associate all the tags to the post
                    foreach (var tag in db.Tags.Where(t => tagsList.Contains(t.Value)))
                    {
                        post.Tags.Add(tag);
                    }
                }


                /* db.Posts.Attach(post);
                 db.ObjectStateManager.ChangeObjectState(post, EntityState.Modified);*/
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.Author_Id = new SelectList(db.Authors, "UserId", "UserName", newPost.Author_Id);
            return View(newPost);
        }

        //
        // GET: /Post/Delete/5
 
        public ActionResult Delete(int id)
        {
            Post post = db.Posts.Single(p => p.Id == id);
            return View(post);
        }

        //
        // POST: /Post/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {            
            Post post = db.Posts.Include("Tags").Single(p => p.Id == id);
            post.Tags.Clear();
            db.Posts.DeleteObject(post);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}